#!/usr/bin/env python
# SPDX-License-Identifier: ISC

#
# Copyright (c) 2021 by VMware, Inc. ("VMware")
# Used Copyright (c) 2018 by Network Device Education Foundation,
# Inc. ("NetDEF") in this file.
#

"""
Following tests are covered to test BGP Multi-VRF Dynamic Route Leaking:
1. Verify that with multiple tenant VRFs, dynamically imported routes are
    further advertised to eBGP peers.
2. Verify the route-map operations along with dynamic import command
3. Verify that deleting static routes from originating VRF also deletes
    routes from other VRFs and peers.
4. Verify that deleting and adding "import" command multiple times shows
    consistent results.
"""

import os
import sys
import time
import pytest
import platform

# Save the Current Working Directory to find configuration files.
CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, "../"))
sys.path.append(os.path.join(CWD, "../lib/"))

# Required to instantiate the topology builder class.

# pylint: disable=C0413
# Import topogen and topotest helpers
from lib.topogen import Topogen, get_topogen
from lib.topotest import version_cmp

from lib.common_config import (
    start_topology,
    write_test_header,
    check_address_types,
    write_test_footer,
    reset_config_on_routers,
    verify_rib,
    step,
    create_route_maps,
    create_static_routes,
    create_prefix_lists,
    create_bgp_community_lists,
)

from lib.topolog import logger
from lib.bgp import (
    verify_bgp_convergence,
    create_router_bgp,
    verify_bgp_community,
    verify_bgp_rib,
)
from lib.topojson import build_config_from_json

pytestmark = [pytest.mark.bgpd, pytest.mark.staticd]

# Global variables
NETWORK1_1 = {"ipv4": "11.11.11.1/32", "ipv6": "11:11::1/128"}
NETWORK1_2 = {"ipv4": "11.11.11.11/32", "ipv6": "11:11::11/128"}
NETWORK1_3 = {"ipv4": "10.10.10.1/32", "ipv6": "10:10::1/128"}
NETWORK1_4 = {"ipv4": "10.10.10.100/32", "ipv6": "10:10::100/128"}
NETWORK1_5 = {"ipv4": "110.110.110.1/32", "ipv6": "110:110::1/128"}
NETWORK1_6 = {"ipv4": "110.110.110.100/32", "ipv6": "110:110::100/128"}

NETWORK2_1 = {"ipv4": "22.22.22.2/32", "ipv6": "22:22::2/128"}
NETWORK2_2 = {"ipv4": "22.22.22.22/32", "ipv6": "22:22::22/128"}
NETWORK2_3 = {"ipv4": "20.20.20.20/32", "ipv6": "20:20::20/128"}
NETWORK2_4 = {"ipv4": "20.20.20.200/32", "ipv6": "20:20::200/128"}
NETWORK2_5 = {"ipv4": "220.220.220.20/32", "ipv6": "220:220::20/128"}
NETWORK2_6 = {"ipv4": "220.220.220.200/32", "ipv6": "220:220::200/128"}

NETWORK3_1 = {"ipv4": "30.30.30.3/32", "ipv6": "30:30::3/128"}
NETWORK3_2 = {"ipv4": "30.30.30.30/32", "ipv6": "30:30::30/128"}

PREFIX_LIST = {
    "ipv4": ["11.11.11.1", "22.22.22.2", "22.22.22.22"],
    "ipv6": ["11:11::1", "22:22::2", "22:22::22"],
}
PREFERRED_NEXT_HOP = "global"
VRF_LIST = ["RED", "BLUE", "GREEN"]
COMM_VAL_1 = "100:100"
COMM_VAL_2 = "500:500"
COMM_VAL_3 = "600:600"


def setup_module(mod):
    """
    Sets up the pytest environment

    * `mod`: module name
    """

    testsuite_run_time = time.asctime(time.localtime(time.time()))
    logger.info("Testsuite start time: {}".format(testsuite_run_time))
    logger.info("=" * 40)

    logger.info("Running setup_module to create topology")

    # This function initiates the topology build with Topogen...
    json_file = "{}/bgp_vrf_dynamic_route_leak_topo3.json".format(CWD)
    tgen = Topogen(json_file, mod.__name__)
    global topo
    topo = tgen.json_topo
    # ... and here it calls Mininet initialization functions.

    # Starting topology, create tmp files which are loaded to routers
    #  to start daemons and then start routers
    start_topology(tgen)

    # Run these tests for kernel version 4.19 or above
    if version_cmp(platform.release(), "4.19") < 0:
        error_msg = (
            "BGP vrf dynamic route leak tests will not run "
            '(have kernel "{}", but it requires >= 4.19)'.format(platform.release())
        )
        pytest.skip(error_msg)

    # Creating configuration from JSON
    build_config_from_json(tgen, topo)

    global BGP_CONVERGENCE
    global ADDR_TYPES
    ADDR_TYPES = check_address_types()

    BGP_CONVERGENCE = verify_bgp_convergence(tgen, topo)
    assert BGP_CONVERGENCE is True, "setup_module : Failed \n Error: {}".format(
        BGP_CONVERGENCE
    )

    logger.info("Running setup_module() done")


def teardown_module():
    """Teardown the pytest environment"""

    logger.info("Running teardown_module to delete topology")

    tgen = get_topogen()

    # Stop toplogy and Remove tmp files
    tgen.stop_topology()

    logger.info(
        "Testsuite end time: {}".format(time.asctime(time.localtime(time.time())))
    )
    logger.info("=" * 40)


#####################################################
#
#   Testcases
#
#####################################################


def test_dynamic_import_routes_advertised_to_ebgp_peers_p0(request):
    """
    Verify that with multiple tenant VRFs, dynamically imported routes are
    further advertised to eBGP peers.
    """

    tgen = get_topogen()
    tc_name = request.node.name
    write_test_header(tc_name)
    reset_config_on_routers(tgen)
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    step(
        "Configure static routes on R2 and R3 and redistribute in BGP for "
        "BLUE and RED vrf instances"
    )
    for dut, network in zip(
        ["r2", "r3"], [[NETWORK1_1, NETWORK1_2], [NETWORK2_1, NETWORK2_2]]
    ):
        for vrf_name, network_vrf in zip(["RED", "BLUE"], network):
            step("Configure static route for VRF : {} on {}".format(vrf_name, dut))
            for addr_type in ADDR_TYPES:
                static_routes = {
                    dut: {
                        "static_routes": [
                            {
                                "network": [network_vrf[addr_type]],
                                "next_hop": "blackhole",
                                "vrf": vrf_name,
                            }
                        ]
                    }
                }

                result = create_static_routes(tgen, static_routes)
                assert result is True, "Testcase {} :Failed \n Error: {}".format(
                    tc_name, result
                )

    for dut, as_num in zip(["r2", "r3"], ["2", "3"]):
        for vrf_name in ["RED", "BLUE"]:
            step("Redistribute static route on BGP VRF : {}".format(vrf_name))
            temp = {}
            for addr_type in ADDR_TYPES:
                temp.update(
                    {
                        addr_type: {
                            "unicast": {"redistribute": [{"redist_type": "static"}]}
                        }
                    }
                )

            redist_dict = {
                dut: {
                    "bgp": [
                        {"vrf": vrf_name, "local_as": as_num, "address_family": temp}
                    ]
                }
            }

            result = create_router_bgp(tgen, topo, redist_dict)
            assert result is True, "Testcase {} :Failed \n Error: {}".format(
                tc_name, result
            )

    step(
        "Verify that R2 and R3 has installed redistributed routes in BLUE "
        "and RED vrfs"
    )
    for dut, network in zip(
        ["r2", "r3"], [[NETWORK2_1, NETWORK2_2], [NETWORK1_1, NETWORK1_2]]
    ):
        for vrf_name, network_vrf in zip(["RED", "BLUE"], network):
            for addr_type in ADDR_TYPES:
                static_routes = {
                    dut: {
                        "static_routes": [
                            {
                                "network": [network_vrf[addr_type]],
                                "next_hop": "blackhole",
                                "vrf": vrf_name,
                            }
                        ]
                    }
                }
                result = verify_bgp_rib(tgen, addr_type, dut, static_routes)
                assert result is True, "Testcase {} : Failed \n Error {}".format(
                    tc_name, result
                )

                result = verify_rib(tgen, addr_type, dut, static_routes)
                assert result is True, "Testcase {} : Failed \n Error {}".format(
                    tc_name, result
                )

    step(
        "Import BLUE vrf's route in tenant vrf RED on R2 and then import "
        "vrf RED's routes into BLUE vrf on R3"
    )

    for dut, as_num, vrf_name, vrf_import in zip(
        ["r2", "r3"], ["2", "3"], ["RED", "BLUE"], ["BLUE", "RED"]
    ):
        step("Import vrf {} int vrf {}, on router {}".format(vrf_import, vrf_name, dut))
        temp = {}
        for addr_type in ADDR_TYPES:
            temp.update({addr_type: {"unicast": {"import": {"vrf": vrf_import}}}})

        import_dict = {
            dut: {
                "bgp": [{"vrf": vrf_name, "local_as": as_num, "address_family": temp}]
            }
        }

        result = create_router_bgp(tgen, topo, import_dict)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

    step(
        "Verify that R2's vrf RED and R3's vrf BLUE has installed 4 set of "
        "prefixes. Routes imported from BLUE vrf (originated R2's & received "
        "from R3). Vrf RED's local routes (originated by R2's & received "
        "from R3)"
    )
    step(
        "Verify that R2 and R3 has installed redistributed routes in BLUE "
        "and RED vrfs"
    )

    for dut, vrf_name in zip(["r2", "r3"], ["RED", "BLUE"]):
        for addr_type in ADDR_TYPES:
            static_routes = {
                dut: {
                    "static_routes": [
                        {
                            "network": [
                                NETWORK1_1[addr_type],
                                NETWORK1_2[addr_type],
                                NETWORK2_1[addr_type],
                                NETWORK2_2[addr_type],
                            ],
                            "next_hop": "blackhole",
                            "vrf": vrf_name,
                        }
                    ]
                }
            }
            result = verify_bgp_rib(tgen, addr_type, dut, static_routes)
            assert result is True, "Testcase {} : Failed \n Error {}".format(
                tc_name, result
            )

            result = verify_rib(tgen, addr_type, dut, static_routes)
            assert result is True, "Testcase {} : Failed \n Error {}".format(
                tc_name, result
            )

    step(
        "Additionally, R2 receives R3's BLUE vrf's prefixes and then import "
        "into vrf RED. These imported routes are advertised back to "
        "(originator)R3 but now in vrf RED, however R3 doesn't install these "
        "in vrf RED. Denied due to own AS"
    )
    for addr_type in ADDR_TYPES:
        static_routes = {
            "r3": {
                "static_routes": [
                    {
                        "network": [
                            NETWORK1_1[addr_type],
                            NETWORK1_2[addr_type],
                            NETWORK2_1[addr_type],
                            NETWORK2_2[addr_type],
                        ],
                        "next_hop": "blackhole",
                        "vrf": "RED",
                    }
                ]
            }
        }
        result = verify_bgp_rib(tgen, addr_type, "r3", static_routes, expected=False)
        assert (
            result is not True
        ), "Testcase {} : Failed \nError {}\n" "Routes {} still in BGP table".format(
            tc_name, result, static_routes["r3"]["static_routes"][0]["network"]
        )

        result = verify_rib(tgen, addr_type, "r3", static_routes, expected=False)
        assert result is not True, "Testcase {} : Failed \n Error {}".format(
            tc_name, result
        )

    step("Remove import vrf BLUE from vrf RED's instance on R2.")
    temp = {}
    for addr_type in ADDR_TYPES:
        temp.update(
            {addr_type: {"unicast": {"import": {"vrf": "BLUE", "delete": True}}}}
        )

    import_dict = {
        "r2": {"bgp": [{"vrf": "RED", "local_as": 2, "address_family": temp}]}
    }

    result = create_router_bgp(tgen, topo, import_dict)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
    step(
        "Verify on R3 that, there is no change in FIB of vrf BLUE and R2's "
        "BLUE vrf originated routes are removed from vrf RED on R3."
    )
    for vrf_name in ["RED", "BLUE"]:
        for addr_type in ADDR_TYPES:
            if vrf_name == "RED":
                network_vrf = [NETWORK1_1[addr_type], NETWORK2_1[addr_type]]
            elif vrf_name == "BLUE":
                network_vrf = [
                    NETWORK1_1[addr_type],
                    NETWORK1_2[addr_type],
                    NETWORK2_1[addr_type],
                    NETWORK2_2[addr_type],
                ]
            static_routes = {
                "r3": {
                    "static_routes": [
                        {
                            "network": network_vrf,
                            "next_hop": "blackhole",
                            "vrf": vrf_name,
                        }
                    ]
                }
            }
            result = verify_bgp_rib(tgen, addr_type, "r3", static_routes)
            assert result is True, "Testcase {} : Failed \n Error {}".format(
                tc_name, result
            )

            result = verify_rib(tgen, addr_type, "r3", static_routes)
            assert result is True, "Testcase {} : Failed \n Error {}".format(
                tc_name, result
            )

    step("Remove import vrf BLUE from vrf RED's instance on R2.")
    temp = {}
    for addr_type in ADDR_TYPES:
        temp.update({addr_type: {"unicast": {"import": {"vrf": "BLUE"}}}})

    import_dict = {
        "r2": {"bgp": [{"vrf": "RED", "local_as": 2, "address_family": temp}]}
    }

    result = create_router_bgp(tgen, topo, import_dict)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "All the routes described in earlier step should be added, once "
        "import command on R2 is re-added."
    )
    for dut, vrf_name in zip(["r2", "r3"], ["RED", "BLUE"]):
        for addr_type in ADDR_TYPES:
            static_routes = {
                dut: {
                    "static_routes": [
                        {
                            "network": [
                                NETWORK1_1[addr_type],
                                NETWORK1_2[addr_type],
                                NETWORK2_1[addr_type],
                                NETWORK2_2[addr_type],
                            ],
                            "next_hop": "blackhole",
                            "vrf": vrf_name,
                        }
                    ]
                }
            }
            result = verify_bgp_rib(tgen, addr_type, dut, static_routes)
            assert result is True, "Testcase {} : Failed \n Error {}".format(
                tc_name, result
            )

            result = verify_rib(tgen, addr_type, dut, static_routes)
            assert result is True, "Testcase {} : Failed \n Error {}".format(
                tc_name, result
            )

    step("Remove import vrf RED from BLUE vrf on R3")
    temp = {}
    for addr_type in ADDR_TYPES:
        temp.update(
            {addr_type: {"unicast": {"import": {"vrf": "RED", "delete": True}}}}
        )

    import_dict = {
        "r3": {"bgp": [{"vrf": "BLUE", "local_as": 3, "address_family": temp}]}
    }

    result = create_router_bgp(tgen, topo, import_dict)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify on R2 that, there is no change in FIB of vrf RED and R3's "
        "vrf RED's originated routes are removed from vrf BLUE on R2."
    )
    for vrf_name in ["RED", "BLUE"]:
        for addr_type in ADDR_TYPES:
            if vrf_name == "BLUE":
                network_vrf = [NETWORK1_2[addr_type], NETWORK2_2[addr_type]]
            elif vrf_name == "RED":
                network_vrf = [
                    NETWORK1_1[addr_type],
                    NETWORK1_2[addr_type],
                    NETWORK2_1[addr_type],
                    NETWORK2_2[addr_type],
                ]
            static_routes = {
                "r2": {
                    "static_routes": [
                        {
                            "network": network_vrf,
                            "next_hop": "blackhole",
                            "vrf": vrf_name,
                        }
                    ]
                }
            }
            result = verify_bgp_rib(tgen, addr_type, "r2", static_routes)
            assert result is True, "Testcase {} : Failed \n Error {}".format(
                tc_name, result
            )

            result = verify_rib(tgen, addr_type, "r2", static_routes)
            assert result is True, "Testcase {} : Failed \n Error {}".format(
                tc_name, result
            )

    step("Add import vrf RED from BLUE vrf on R3")
    temp = {}
    for addr_type in ADDR_TYPES:
        temp.update({addr_type: {"unicast": {"import": {"vrf": "RED"}}}})

    import_dict = {
        "r3": {"bgp": [{"vrf": "BLUE", "local_as": 3, "address_family": temp}]}
    }

    result = create_router_bgp(tgen, topo, import_dict)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "All the routes described in earlier step should be added, once "
        "import command on R2 is re-added."
    )
    for dut, vrf_name in zip(["r2", "r3"], ["RED", "BLUE"]):
        for addr_type in ADDR_TYPES:
            static_routes = {
                dut: {
                    "static_routes": [
                        {
                            "network": [
                                NETWORK1_1[addr_type],
                                NETWORK1_2[addr_type],
                                NETWORK2_1[addr_type],
                                NETWORK2_2[addr_type],
                            ],
                            "next_hop": "blackhole",
                            "vrf": vrf_name,
                        }
                    ]
                }
            }
            result = verify_bgp_rib(tgen, addr_type, dut, static_routes)
            assert result is True, "Testcase {} : Failed \n Error {}".format(
                tc_name, result
            )

            result = verify_rib(tgen, addr_type, dut, static_routes)
            assert result is True, "Testcase {} : Failed \n Error {}".format(
                tc_name, result
            )

    write_test_footer(tc_name)


def test_dynamic_imported_matching_prefix_based_on_community_list_p0(request):
    """
    Verify the route-map operations along with dynamic import command
    """

    tgen = get_topogen()
    tc_name = request.node.name
    write_test_header(tc_name)
    reset_config_on_routers(tgen)
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    step(
        "Configure static routes on R3 for vrf RED and redistribute in BGP " "instance"
    )
    for vrf_name, networks in zip(
        ["RED", "BLUE"], [[NETWORK1_1, NETWORK1_2], [NETWORK2_1, NETWORK2_2]]
    ):
        for addr_type in ADDR_TYPES:
            static_routes = {
                "r3": {
                    "static_routes": [
                        {
                            "network": [networks[0][addr_type], networks[1][addr_type]],
                            "next_hop": "blackhole",
                            "vrf": vrf_name,
                        }
                    ]
                }
            }

            result = create_static_routes(tgen, static_routes)
            assert result is True, "Testcase {} :Failed \n Error: {}".format(
                tc_name, result
            )

    step(
        "Configure route-map to set community attribute for a specific " "prefix on R3"
    )
    for addr_type in ADDR_TYPES:
        input_dict_pf = {
            "r3": {
                "prefix_lists": {
                    addr_type: {
                        "pflist_ABC_{}".format(addr_type): [
                            {
                                "seqid": 10,
                                "network": NETWORK1_1[addr_type],
                                "action": "permit",
                            }
                        ]
                    }
                }
            }
        }
        result = create_prefix_lists(tgen, input_dict_pf)
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )

    input_dict_cl = {
        "r3": {
            "bgp_community_lists": [
                {
                    "community_type": "expanded",
                    "action": "permit",
                    "name": "COMM",
                    "value": COMM_VAL_1,
                }
            ]
        }
    }
    result = create_bgp_community_lists(tgen, input_dict_cl)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    for addr_type in ADDR_TYPES:
        input_dict_rm = {
            "r3": {
                "route_maps": {
                    "rmap_XYZ_{}".format(addr_type): [
                        {
                            "action": "permit",
                            "match": {
                                addr_type: {
                                    "prefix_lists": "pflist_ABC_{}".format(addr_type)
                                }
                            },
                            "set": {"community": {"num": COMM_VAL_1}},
                        }
                    ]
                }
            }
        }
        result = create_route_maps(tgen, input_dict_rm)
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )

    step(
        "Apply this route-map on R3 to set community under vrf RED/BLUE "
        "while redistributing the prefixes into BGP"
    )
    temp = {}
    for addr_type in ADDR_TYPES:
        temp.update(
            {
                addr_type: {
                    "unicast": {
                        "redistribute": [
                            {
                                "redist_type": "static",
                                "attribute": {
                                    "route-map": "rmap_XYZ_{}".format(addr_type)
                                },
                            }
                        ]
                    }
                }
            }
        )

    for vrf_name in ["RED", "BLUE"]:
        redist_dict = {
            "r3": {"bgp": [{"vrf": vrf_name, "local_as": 3, "address_family": temp}]}
        }

        result = create_router_bgp(tgen, topo, redist_dict)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

    step(
        "Verify that specific prefixes matched in route-map have community "
        "attribute value 100:100 tagged"
    )
    input_dict_comm = {"community": COMM_VAL_1}
    for addr_type in ADDR_TYPES:
        result = verify_bgp_community(
            tgen, addr_type, "r3", [NETWORK1_1[addr_type]], input_dict_comm, vrf="RED"
        )
        assert result is True, "Testcase {} : Failed \n Error {}".format(
            tc_name, result
        )

    step(
        "Configure a route-map for filtering the prefixes based on community "
        "attribute while importing into default vrf"
    )
    for addr_type in ADDR_TYPES:
        input_dict_rm = {
            "r3": {
                "route_maps": {
                    "rmap_IMP_{}".format(addr_type): [
                        {
                            "action": "permit",
                            "seq_id": 10,
                            "match": {"community_list": {"id": "COMM"}},
                            "set": {"community": {"num": COMM_VAL_2}},
                        }
                    ]
                }
            }
        }
        result = create_route_maps(tgen, input_dict_rm)
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )

    step(
        "Apply the route-map while Importing vrf RED/BLUE's prefixes into "
        "GREEN vrf on router R3"
    )
    temp = {}
    for vrf_name in ["RED", "BLUE"]:
        for addr_type in ADDR_TYPES:
            temp.update({addr_type: {"unicast": {"import": {"vrf": vrf_name}}}})

        inport_dict = {
            "r3": {"bgp": [{"vrf": "GREEN", "local_as": 3, "address_family": temp}]}
        }

        result = create_router_bgp(tgen, topo, inport_dict)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

    temp = {}
    for addr_type in ADDR_TYPES:
        temp.update(
            {
                addr_type: {
                    "unicast": {
                        "import": {"vrf": "route-map rmap_IMP_{}".format(addr_type)}
                    }
                }
            }
        )

    inport_dict = {
        "r3": {"bgp": [{"vrf": "GREEN", "local_as": 3, "address_family": temp}]}
    }

    result = create_router_bgp(tgen, topo, inport_dict)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    input_dict_comm = {"community": COMM_VAL_2}
    step(
        "Verify on R3 that only prefixes with community value {} in vrf RED "
        "are imported to vrf GREEN. While importing, the community value "
        "has been changed to {}".format(COMM_VAL_1, COMM_VAL_2)
    )

    for addr_type in ADDR_TYPES:
        static_routes = {
            "r3": {
                "static_routes": [{"network": [NETWORK1_1[addr_type]], "vrf": "GREEN"}]
            }
        }

        result = verify_bgp_rib(tgen, addr_type, "r3", static_routes)
        assert result is True, "Testcase {} : Failed \n Error {}".format(
            tc_name, result
        )

        result = verify_rib(tgen, addr_type, "r3", static_routes)
        assert result is True, "Testcase {} : Failed \n Error {}".format(
            tc_name, result
        )

        static_routes = {
            "r3": {
                "static_routes": [
                    {
                        "network": [
                            NETWORK2_1[addr_type],
                            NETWORK2_2[addr_type],
                            NETWORK1_2[addr_type],
                        ],
                        "vrf": "GREEN",
                    }
                ]
            }
        }

        result = verify_bgp_rib(tgen, addr_type, "r3", static_routes, expected=False)
        assert (
            result is not True
        ), "Testcase {} : Failed \nError {}\n" "Routes {} still in BGP table".format(
            tc_name, result, static_routes["r3"]["static_routes"][0]["network"]
        )

        result = verify_rib(tgen, addr_type, "r3", static_routes, expected=False)
        assert (
            result is not True
        ), "Testcase {} : Failed Error {}" "Routes {} still in Route table".format(
            tc_name, result, static_routes["r3"]["static_routes"][0]["network"]
        )

        result = verify_bgp_community(
            tgen, addr_type, "r3", [NETWORK1_1[addr_type]], input_dict_comm, vrf="GREEN"
        )
        assert result is True, "Testcase {} : Failed \n Error {}".format(
            tc_name, result
        )

    for action, value in zip(["Delete", "Add"], [True, False]):
        step("{} import vrf RED/BLUE command one by one from vrf GREEN".format(action))
        temp = {}
        for vrf_name in ["RED", "BLUE"]:
            for addr_type in ADDR_TYPES:
                temp.update(
                    {
                        addr_type: {
                            "unicast": {"import": {"vrf": vrf_name, "delete": value}}
                        }
                    }
                )

            inport_dict = {
                "r3": {"bgp": [{"vrf": "GREEN", "local_as": 3, "address_family": temp}]}
            }

            result = create_router_bgp(tgen, topo, inport_dict)
            assert result is True, "Testcase {} :Failed \n Error: {}".format(
                tc_name, result
            )

        step(
            "Verify that when import vrf RED/BLUE is {} one by one, all "
            "routes of respective vrf disappear from vrf GREEN without "
            "affecting (BLUE/RED) routes".format(action.lower())
        )
        for addr_type in ADDR_TYPES:
            static_routes = {
                "r3": {
                    "static_routes": [
                        {"network": [NETWORK1_1[addr_type]], "vrf": "GREEN"}
                    ]
                }
            }

            if value:
                result = verify_bgp_rib(
                    tgen, addr_type, "r3", static_routes, expected=False
                )
                assert result is not True, (
                    "Testcase {} : Failed \nError {}\n"
                    "Routes {} still in BGP table".format(
                        tc_name,
                        result,
                        static_routes["r3"]["static_routes"][0]["network"],
                    )
                )

                result = verify_rib(
                    tgen, addr_type, "r3", static_routes, expected=False
                )
                assert result is not True, (
                    "Testcase {} : Failed Error {}"
                    "Routes {} still in Route table".format(
                        tc_name,
                        result,
                        static_routes["r3"]["static_routes"][0]["network"],
                    )
                )
            else:
                result = verify_bgp_rib(tgen, addr_type, "r3", static_routes)
                assert result is True, "Testcase {} : Failed \n Error {}".format(
                    tc_name, result
                )

                result = verify_rib(tgen, addr_type, "r3", static_routes)
                assert result is True, "Testcase {} : Failed \n Error {}".format(
                    tc_name, result
                )

    for action, value in zip(["Delete", "Re-add"], [True, False]):
        step(
            "{} route-map IMP from global config when import and route-maps "
            "are applied in vrf GREEN".format(action)
        )
        for addr_type in ADDR_TYPES:
            input_dict_rm = {
                "r3": {
                    "route_maps": {
                        "rmap_IMP_{}".format(addr_type): [
                            {
                                "action": "permit",
                                "seq_id": 10,
                                "match": {"community_list": {"id": "COMM"}},
                                "set": {"community": {"num": COMM_VAL_2}},
                                "delete": value,
                            }
                        ]
                    }
                }
            }
            result = create_route_maps(tgen, input_dict_rm)
            assert result is True, "Testcase {} : Failed \n Error: {}".format(
                tc_name, result
            )

        step(
            "Verify that when import vrf RED/BLUE is {} one by one, all "
            "routes of respective vrf disappear from vrf GREEN without "
            "affecting (BLUE/RED) routes".format(action.lower())
        )
        for addr_type in ADDR_TYPES:
            static_routes = {
                "r3": {
                    "static_routes": [
                        {"network": [NETWORK1_1[addr_type]], "vrf": "GREEN"}
                    ]
                }
            }

            if value:
                result = verify_bgp_rib(
                    tgen, addr_type, "r3", static_routes, expected=False
                )
                assert result is not True, (
                    "Testcase {} : Failed \nError {}\n"
                    "Routes {} still in BGP table".format(
                        tc_name,
                        result,
                        static_routes["r3"]["static_routes"][0]["network"],
                    )
                )

                result = verify_rib(
                    tgen, addr_type, "r3", static_routes, expected=False
                )
                assert result is not True, (
                    "Testcase {} : Failed Error {}"
                    "Routes {} still in Route table".format(
                        tc_name,
                        result,
                        static_routes["r3"]["static_routes"][0]["network"],
                    )
                )
            else:
                result = verify_bgp_rib(tgen, addr_type, "r3", static_routes)
                assert result is True, "Testcase {} : Failed \n Error {}".format(
                    tc_name, result
                )

                result = verify_rib(tgen, addr_type, "r3", static_routes)
                assert result is True, "Testcase {} : Failed \n Error {}".format(
                    tc_name, result
                )

    write_test_footer(tc_name)


def test_dynamic_import_routes_delete_static_route_p1(request):
    """
    Verify that deleting static routes from originating VRF also deletes
    routes from other VRFs and peers.
    """

    tgen = get_topogen()
    tc_name = request.node.name
    write_test_header(tc_name)
    reset_config_on_routers(tgen)
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    step(
        "Configure static routes on R3 for each tenant vrf and redistribute "
        "in respective BGP instance"
    )
    vrf_list = VRF_LIST + ["default"]
    for vrf_name, network in zip(
        vrf_list, [NETWORK1_1, NETWORK2_1, NETWORK3_1, NETWORK1_2]
    ):
        step("Configure static route for VRF : {}".format(vrf_name))
        for addr_type in ADDR_TYPES:
            static_routes = {
                "r3": {
                    "static_routes": [
                        {
                            "network": [network[addr_type]],
                            "next_hop": "blackhole",
                            "vrf": vrf_name,
                        }
                    ]
                }
            }

            result = create_static_routes(tgen, static_routes)
            assert result is True, "Testcase {} :Failed \n Error: {}".format(
                tc_name, result
            )

        step("Redistribute static route on BGP VRF : {}".format(vrf_name))
        temp = {}
        for addr_type in ADDR_TYPES:
            temp.update(
                {addr_type: {"unicast": {"redistribute": [{"redist_type": "static"}]}}}
            )

        redist_dict = {
            "r3": {"bgp": [{"vrf": vrf_name, "local_as": 3, "address_family": temp}]}
        }

        result = create_router_bgp(tgen, topo, redist_dict)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

    for vrf_name, network in zip(vrf_list, [NETWORK1_1, NETWORK2_1, NETWORK3_1]):
        step(
            "Verify that R3 has installed redistributed routes in respective "
            "vrfs: {}".format(vrf_name)
        )
        for addr_type in ADDR_TYPES:
            static_routes = {
                "r3": {
                    "static_routes": [
                        {
                            "network": [network[addr_type]],
                            "next_hop": "blackhole",
                            "vrf": vrf_name,
                        }
                    ]
                }
            }

            result = verify_rib(tgen, addr_type, "r3", static_routes)
            assert result is True, "Testcase {} : Failed \n Error {}".format(
                tc_name, result
            )

    step("Import routes among vrfs as mentioned below on router R3")

    for vrf_name, vrf_import in zip(
        ["GREEN", "BLUE", "default"], ["RED", "GREEN", "BLUE"]
    ):
        temp = {}
        for addr_type in ADDR_TYPES:
            temp.update({addr_type: {"unicast": {"import": {"vrf": vrf_import}}}})

        import_dict = {
            "r3": {"bgp": [{"vrf": vrf_name, "local_as": 3, "address_family": temp}]}
        }

        result = create_router_bgp(tgen, topo, import_dict)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

    for vrf_name, vrf_import, installed, not_installed in zip(
        ["BLUE", "default"],
        ["GREEN", "BLUE"],
        [NETWORK3_1, NETWORK2_1],
        [NETWORK1_1, NETWORK3_1],
    ):
        step(
            "Verify that only locally originated routes of vrf {} are "
            "advertised to vrf {}".format(vrf_import, vrf_name)
        )

        for addr_type in ADDR_TYPES:
            static_routes = {
                "r3": {
                    "static_routes": [
                        {"network": [installed[addr_type]], "vrf": vrf_name}
                    ]
                }
            }
            result = verify_bgp_rib(tgen, addr_type, "r2", static_routes)
            assert result is True, "Testcase {} : Failed \n Error {}".format(
                tc_name, result
            )

            result = verify_rib(tgen, addr_type, "r2", static_routes)
            assert result is True, "Testcase {} : Failed \n Error {}".format(
                tc_name, result
            )

            step(
                "Verify that non local originated routes {} of vrf {} are "
                "not advertised to vrf {}".format(
                    not_installed[addr_type], vrf_import, vrf_name
                )
            )

            static_routes = {
                "r3": {
                    "static_routes": [
                        {"network": [not_installed[addr_type]], "vrf": vrf_name}
                    ]
                }
            }
            result = verify_bgp_rib(
                tgen, addr_type, "r2", static_routes, expected=False
            )
            assert result is not True, (
                "Testcase {} : Failed \nError {}\n"
                "Routes {} still in BGP table".format(
                    tc_name, result, static_routes["r2"]["static_routes"][0]["network"]
                )
            )

            result = verify_rib(tgen, addr_type, "r2", static_routes, expected=False)
            assert (
                result is not True
            ), "Testcase {} : Failed Error {}" "Routes {} still in Route table".format(
                tc_name, result, static_routes["r2"]["static_routes"][0]["network"]
            )

    step("Delete static routes from vrf RED")
    for addr_type in ADDR_TYPES:
        static_routes = {
            "r3": {
                "static_routes": [
                    {
                        "network": [NETWORK1_1[addr_type]],
                        "next_hop": "blackhole",
                        "vrf": "RED",
                        "delete": True,
                    }
                ]
            }
        }

        result = create_static_routes(tgen, static_routes)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )
    step(
        "Verify on R2 and R3, that only vrf RED and GREEN's RIB/FIB withdraw "
        "deleted routes"
    )
    for dut in ["r2", "r3"]:
        step(
            "Verify on {}, that only vrf RED and GREEN's RIB/FIB withdraw "
            "deleted routes".format(dut)
        )
        for vrf_name in ["RED", "GREEN"]:
            for addr_type in ADDR_TYPES:
                static_routes = {
                    "r3": {
                        "static_routes": [
                            {"network": [NETWORK1_1[addr_type]], "vrf": vrf_name}
                        ]
                    }
                }
                result = verify_bgp_rib(
                    tgen, addr_type, "r2", static_routes, expected=False
                )
                assert result is not True, (
                    "Testcase {} : Failed \nError {}\n"
                    "Routes {} still in BGP table".format(
                        tc_name,
                        result,
                        static_routes["r2"]["static_routes"][0]["network"],
                    )
                )

                result = verify_rib(
                    tgen, addr_type, "r2", static_routes, expected=False
                )
                assert result is not True, (
                    "Testcase {} : Failed Error {}"
                    "Routes {} still in Route table".format(
                        tc_name,
                        result,
                        static_routes[dut]["static_routes"][0]["network"],
                    )
                )

    step("Delete static routes from vrf BLUE")
    for addr_type in ADDR_TYPES:
        static_routes = {
            "r3": {
                "static_routes": [
                    {
                        "network": [NETWORK2_1[addr_type]],
                        "next_hop": "blackhole",
                        "vrf": "BLUE",
                        "delete": True,
                    }
                ]
            }
        }

        result = create_static_routes(tgen, static_routes)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

    for dut in ["r2", "r3"]:
        step(
            "Verify on {}, that only default and BLUE vrf's RIB/FIB "
            "withdraw deleted routes".format(dut)
        )
        for vrf_name in ["BLUE", "default"]:
            for addr_type in ADDR_TYPES:
                static_routes = {
                    "r3": {
                        "static_routes": [
                            {"network": [NETWORK2_1[addr_type]], "vrf": vrf_name}
                        ]
                    }
                }
                result = verify_bgp_rib(
                    tgen, addr_type, dut, static_routes, expected=False
                )
                assert result is not True, (
                    "Testcase {} : Failed \nError {}\n"
                    "Routes {} still in BGP table".format(
                        tc_name,
                        result,
                        static_routes[dut]["static_routes"][0]["network"],
                    )
                )

                result = verify_rib(tgen, addr_type, dut, static_routes, expected=False)
                assert result is not True, (
                    "Testcase {} : Failed Error {}"
                    "Routes {} still in Route table".format(
                        tc_name,
                        result,
                        static_routes[dut]["static_routes"][0]["network"],
                    )
                )

    step("Delete static routes from vrf default")
    for addr_type in ADDR_TYPES:
        static_routes = {
            "r3": {
                "static_routes": [
                    {
                        "network": [NETWORK1_2[addr_type]],
                        "next_hop": "blackhole",
                        "delete": True,
                    }
                ]
            }
        }

        result = create_static_routes(tgen, static_routes)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

    for dut in ["r2", "r3"]:
        step(
            "Verify on {}, that only default vrf RIB/FIB withdraw deleted "
            "routes".format(dut)
        )
        for addr_type in ADDR_TYPES:
            static_routes = {
                "r3": {
                    "static_routes": [
                        {"network": [NETWORK1_2[addr_type]], "vrf": vrf_name}
                    ]
                }
            }
            result = verify_bgp_rib(tgen, addr_type, dut, static_routes, expected=False)
            assert result is not True, (
                "Testcase {} : Failed \nError {}\n"
                "Routes {} still in BGP table".format(
                    tc_name, result, static_routes[dut]["static_routes"][0]["network"]
                )
            )

            result = verify_rib(tgen, addr_type, dut, static_routes, expected=False)
            assert result is not True, "Testcase {} : Failed \n Error {}".format(
                tc_name, result
            )

    step("Add back all the routes that were deleted")
    for vrf_name, network in zip(
        vrf_list, [NETWORK1_1, NETWORK2_1, NETWORK3_1, NETWORK1_2]
    ):
        step("Configure static route for VRF : {}".format(vrf_name))
        for addr_type in ADDR_TYPES:
            static_routes = {
                "r3": {
                    "static_routes": [
                        {
                            "network": [network[addr_type]],
                            "next_hop": "blackhole",
                            "vrf": vrf_name,
                        }
                    ]
                }
            }

            result = create_static_routes(tgen, static_routes)
            assert result is True, "Testcase {} :Failed \n Error: {}".format(
                tc_name, result
            )

        step("Redistribute static route on BGP VRF : {}".format(vrf_name))
        temp = {}
        for addr_type in ADDR_TYPES:
            temp.update(
                {addr_type: {"unicast": {"redistribute": [{"redist_type": "static"}]}}}
            )

        redist_dict = {
            "r3": {"bgp": [{"vrf": vrf_name, "local_as": 3, "address_family": temp}]}
        }

        result = create_router_bgp(tgen, topo, redist_dict)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

    for vrf_name, network in zip(vrf_list, [NETWORK1_1, NETWORK2_1, NETWORK3_1]):
        step(
            "Verify that R3 has installed redistributed routes in respective "
            "vrfs: {}".format(vrf_name)
        )
        for addr_type in ADDR_TYPES:
            static_routes = {
                "r3": {
                    "static_routes": [
                        {
                            "network": [network[addr_type]],
                            "next_hop": "blackhole",
                            "vrf": vrf_name,
                        }
                    ]
                }
            }

            result = verify_rib(tgen, addr_type, "r3", static_routes)
            assert result is True, "Testcase {} : Failed \n Error {}".format(
                tc_name, result
            )

    write_test_footer(tc_name)


def test_dynamic_import_routes_add_delete_import_command_p1(request):
    """
    Verify that deleting and adding "import" command multiple times shows
    consistent results.
    """

    tgen = get_topogen()
    tc_name = request.node.name
    write_test_header(tc_name)
    reset_config_on_routers(tgen)
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    step(
        "Configure static routes on R2 for vrf RED and redistribute in "
        "respective BGP instance"
    )
    for addr_type in ADDR_TYPES:
        static_routes = {
            "r2": {
                "static_routes": [
                    {
                        "network": [NETWORK2_1[addr_type]],
                        "next_hop": "blackhole",
                        "vrf": "RED",
                    }
                ]
            }
        }

        result = create_static_routes(tgen, static_routes)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

    step("Redistribute static route on BGP VRF RED")
    temp = {}
    for addr_type in ADDR_TYPES:
        temp.update(
            {addr_type: {"unicast": {"redistribute": [{"redist_type": "static"}]}}}
        )

    redist_dict = {
        "r2": {"bgp": [{"vrf": "RED", "local_as": 2, "address_family": temp}]}
    }

    result = create_router_bgp(tgen, topo, redist_dict)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Verify that R2 has installed redistributed routes in respective " "vrfs only")
    for addr_type in ADDR_TYPES:
        static_routes = {
            "r2": {
                "static_routes": [{"network": [NETWORK2_1[addr_type]], "vrf": "RED"}]
            }
        }
        result = verify_bgp_rib(tgen, addr_type, "r2", static_routes)
        assert result is True, "Testcase {} : Failed \n Error {}".format(
            tc_name, result
        )

        result = verify_rib(tgen, addr_type, "r2", static_routes)
        assert result is True, "Testcase {} : Failed \n Error {}".format(
            tc_name, result
        )

    step("Import vrf RED's routes into vrf GREEN on R2")
    temp = {}
    for addr_type in ADDR_TYPES:
        temp.update({addr_type: {"unicast": {"import": {"vrf": "RED"}}}})

    import_dict = {
        "r2": {"bgp": [{"vrf": "GREEN", "local_as": 2, "address_family": temp}]}
    }
    result = create_router_bgp(tgen, topo, import_dict)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify on R2, that it installs imported routes from vrf RED to vrf "
        "GREEN's RIB/FIB"
    )
    for addr_type in ADDR_TYPES:
        static_routes = {
            "r2": {
                "static_routes": [{"network": [NETWORK2_1[addr_type]], "vrf": "GREEN"}]
            }
        }
        result = verify_bgp_rib(tgen, addr_type, "r2", static_routes)
        assert result is True, "Testcase {} : Failed \n Error {}".format(
            tc_name, result
        )

        result = verify_rib(tgen, addr_type, "r2", static_routes)
        assert result is True, "Testcase {} : Failed \n Error {}".format(
            tc_name, result
        )

    step("On R3 import routes from vrfs GREEN to default")
    temp = {}
    for addr_type in ADDR_TYPES:
        temp.update({addr_type: {"unicast": {"import": {"vrf": "GREEN"}}}})

    import_dict = {"r3": {"bgp": [{"local_as": 3, "address_family": temp}]}}
    result = create_router_bgp(tgen, topo, import_dict)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify that R2's vrf RED routes are now imported into vrf default "
        "of R3, next-hop pointing to vrf GREEN"
    )
    for addr_type in ADDR_TYPES:
        static_routes = {
            "r3": {"static_routes": [{"network": [NETWORK2_1[addr_type]]}]}
        }

        next_hop_1 = topo["routers"]["r2"]["links"]["r3-link3"][addr_type].split("/")[0]
        result = verify_bgp_rib(
            tgen, addr_type, "r3", static_routes, next_hop=next_hop_1
        )
        assert result is True, "Testcase {} : Failed \n Error {}".format(
            tc_name, result
        )

        result = verify_rib(tgen, addr_type, "r3", static_routes, next_hop=next_hop_1)
        assert result is True, "Testcase {} : Failed \n Error {}".format(
            tc_name, result
        )

    step(
        "Delete import command from R3's default vrf instance for both "
        "address-families 1 by 1 (ipv4/ipv6)"
    )
    temp = {}
    for addr_type in ADDR_TYPES:
        temp.update(
            {addr_type: {"unicast": {"import": {"vrf": "GREEN", "delete": True}}}}
        )

    import_dict = {"r3": {"bgp": [{"local_as": 3, "address_family": temp}]}}
    result = create_router_bgp(tgen, topo, import_dict)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify that R2's vrf RED routes are now removed from vrf "
        "default on R3, however vrf GREEN still retains those"
    )
    for addr_type in ADDR_TYPES:
        static_routes = {
            "r3": {"static_routes": [{"network": [NETWORK2_1[addr_type]]}]}
        }
        result = verify_bgp_rib(tgen, addr_type, "r3", static_routes, expected=False)
        assert (
            result is not True
        ), "Testcase {} : Failed \nError {}\n" "Routes {} still in BGP table".format(
            tc_name, result, static_routes["r3"]["static_routes"][0]["network"]
        )

        result = verify_rib(tgen, addr_type, "r3", static_routes, expected=False)
        assert result is not True, "Testcase {} : Failed \n Error {}".format(
            tc_name, result
        )

    step(
        "Delete import command from R2's vrf GREEN instance for both "
        "address-families 1 by 1 (ipv4/ipv6)"
    )
    temp = {}
    for addr_type in ADDR_TYPES:
        temp.update(
            {addr_type: {"unicast": {"import": {"vrf": "RED", "delete": True}}}}
        )

    import_dict = {
        "r2": {"bgp": [{"vrf": "GREEN", "local_as": 2, "address_family": temp}]}
    }
    result = create_router_bgp(tgen, topo, import_dict)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
    step(
        "Verify that R2's vrf RED routes are now removed from vrf GREEN "
        "on R2 & R3 as well"
    )
    for dut in ["r2", "r3"]:
        for addr_type in ADDR_TYPES:
            static_routes = {
                "r2": {
                    "static_routes": [
                        {"network": [NETWORK2_1[addr_type]], "vrf": "GREEN"}
                    ]
                }
            }
            result = verify_bgp_rib(tgen, addr_type, dut, static_routes, expected=False)
            assert result is not True, (
                "Testcase {} : Failed \nError {}\n"
                "Routes {} still in BGP table".format(
                    tc_name, result, static_routes[dut]["static_routes"][0]["network"]
                )
            )

            result = verify_rib(tgen, addr_type, dut, static_routes, expected=False)
            assert (
                result is not True
            ), "Testcase {} : Failed Error {}" "Routes {} still in Route table".format(
                tc_name, result, static_routes[dut]["static_routes"][0]["network"]
            )

    step(
        "Add import command from R3's default vrf instance for both "
        "address-families 1 by 1 (ipv4/ipv6)"
    )
    temp = {}
    for addr_type in ADDR_TYPES:
        temp.update({addr_type: {"unicast": {"import": {"vrf": "GREEN"}}}})

    import_dict = {"r3": {"bgp": [{"local_as": 3, "address_family": temp}]}}
    result = create_router_bgp(tgen, topo, import_dict)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Verify that there are no routes installed on R3's vrf default " "RIB/FIB.")
    for addr_type in ADDR_TYPES:
        static_routes = {
            "r3": {"static_routes": [{"network": [NETWORK2_1[addr_type]]}]}
        }
        result = verify_bgp_rib(tgen, addr_type, "r3", static_routes, expected=False)
        assert (
            result is not True
        ), "Testcase {} : Failed \nError {}\n" "Routes {} still in BGP table".format(
            tc_name, result, static_routes["r3"]["static_routes"][0]["network"]
        )

        result = verify_rib(tgen, addr_type, "r3", static_routes, expected=False)
        assert result is not True, "Testcase {} : Failed \n Error {}".format(
            tc_name, result
        )

    step(
        "Add import command from R2's vrf GREEN instance for both "
        "address-families 1 by 1 (ipv4/ipv6)."
    )
    temp = {}
    for addr_type in ADDR_TYPES:
        temp.update({addr_type: {"unicast": {"import": {"vrf": "RED"}}}})

    import_dict = {
        "r2": {"bgp": [{"vrf": "GREEN", "local_as": 2, "address_family": temp}]}
    }
    result = create_router_bgp(tgen, topo, import_dict)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify that R2's vrf RED routes are now imported into vrf "
        "default of R3, next-hop pointing to vrf GREEN"
    )
    for dut in ["r2", "r3"]:
        for addr_type in ADDR_TYPES:
            static_routes = {
                "r2": {
                    "static_routes": [
                        {"network": [NETWORK2_1[addr_type]], "vrf": "GREEN"}
                    ]
                }
            }
            if dut == "r3":
                next_hop_1 = topo["routers"]["r2"]["links"]["r3-link3"][
                    addr_type
                ].split("/")[0]
                result = verify_bgp_rib(
                    tgen, addr_type, dut, static_routes, next_hop=next_hop_1
                )
            else:
                result = verify_bgp_rib(tgen, addr_type, dut, static_routes)
            assert result is True, "Testcase {} : Failed \n Error {}".format(
                tc_name, result
            )

            if dut == "r3":
                result = verify_rib(
                    tgen, addr_type, dut, static_routes, next_hop=next_hop_1
                )
            else:
                result = verify_rib(tgen, addr_type, dut, static_routes)

            assert result is True, "Testcase {} : Failed \n Error {}".format(
                tc_name, result
            )

    step(
        "Delete import command from R3's default vrf instance for both "
        "address-families 1 by 1 (ipv4/ipv6)."
    )
    temp = {}
    for addr_type in ADDR_TYPES:
        temp.update(
            {addr_type: {"unicast": {"import": {"vrf": "GREEN", "delete": True}}}}
        )

    import_dict = {"r3": {"bgp": [{"local_as": 3, "address_family": temp}]}}
    result = create_router_bgp(tgen, topo, import_dict)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify that R2's vrf RED routes are now removed from vrf "
        "default on R3, however vrf GREEN still retains those."
    )
    for addr_type in ADDR_TYPES:
        static_routes = {
            "r2": {
                "static_routes": [{"network": [NETWORK2_1[addr_type]], "vrf": "GREEN"}]
            }
        }
        result = verify_bgp_rib(tgen, addr_type, "r3", static_routes)
        assert result is True, "Testcase {} : Failed \n Error {}".format(
            tc_name, result
        )

        result = verify_rib(tgen, addr_type, "r3", static_routes)
        assert result is True, "Testcase {} : Failed \n Error {}".format(
            tc_name, result
        )

        static_routes = {
            "r2": {"static_routes": [{"network": [NETWORK2_1[addr_type]]}]}
        }
        result = verify_bgp_rib(tgen, addr_type, "r3", static_routes, expected=False)
        assert (
            result is not True
        ), "Testcase {} : Failed \nError {}\n" "Routes {} still in BGP table".format(
            tc_name, result, static_routes["r3"]["static_routes"][0]["network"]
        )

        result = verify_rib(tgen, addr_type, "r3", static_routes, expected=False)
        assert result is not True, "Testcase {} : Failed \n Error {}".format(
            tc_name, result
        )

    step("Delete redistribute static from R2 for vrf RED")
    temp = {}
    for addr_type in ADDR_TYPES:
        temp.update(
            {
                addr_type: {
                    "unicast": {
                        "redistribute": [{"redist_type": "static", "delete": True}]
                    }
                }
            }
        )

    redist_dict = {
        "r2": {"bgp": [{"vrf": "RED", "local_as": 2, "address_family": temp}]}
    }

    result = create_router_bgp(tgen, topo, redist_dict)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify that R2's vrf RED routes are now removed from vrf GREEN "
        "on R2 & R3 as well."
    )
    for dut in ["r2", "r3"]:
        for addr_type in ADDR_TYPES:
            static_routes = {
                "r2": {
                    "static_routes": [
                        {"network": [NETWORK2_1[addr_type]], "vrf": "GREEN"}
                    ]
                }
            }
            result = verify_bgp_rib(tgen, addr_type, dut, static_routes, expected=False)
            assert result is not True, (
                "Testcase {} : Failed \nError {}\n"
                "Routes {} still in BGP table".format(
                    tc_name, result, static_routes[dut]["static_routes"][0]["network"]
                )
            )

            result = verify_rib(tgen, addr_type, dut, static_routes, expected=False)
            assert (
                result is not True
            ), "Testcase {} : Failed Error {}" "Routes {} still in Route table".format(
                tc_name, result, static_routes[dut]["static_routes"][0]["network"]
            )

    step(
        "Add import command from R3's default vrf instance for both "
        "address-families 1 by 1 (ipv4/ipv6)."
    )
    temp = {}
    for addr_type in ADDR_TYPES:
        temp.update({addr_type: {"unicast": {"import": {"vrf": "GREEN"}}}})

    import_dict = {"r3": {"bgp": [{"local_as": 3, "address_family": temp}]}}
    result = create_router_bgp(tgen, topo, import_dict)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Verify that there are no routes installed on R3's vrf default " "RIB/FIB")
    for addr_type in ADDR_TYPES:
        static_routes = {
            "r3": {"static_routes": [{"network": [NETWORK2_1[addr_type]]}]}
        }
        result = verify_bgp_rib(tgen, addr_type, "r3", static_routes, expected=False)
        assert (
            result is not True
        ), "Testcase {} : Failed \nError {}\n" "Routes {} still in BGP table".format(
            tc_name, result, static_routes["r3"]["static_routes"][0]["network"]
        )

        result = verify_rib(tgen, addr_type, "r3", static_routes, expected=False)
        assert result is not True, "Testcase {} : Failed \n Error {}".format(
            tc_name, result
        )

    step("Add redistribute static from R2 for vrf RED")
    temp = {}
    for addr_type in ADDR_TYPES:
        temp.update(
            {addr_type: {"unicast": {"redistribute": [{"redist_type": "static"}]}}}
        )

    redist_dict = {
        "r2": {"bgp": [{"vrf": "RED", "local_as": 2, "address_family": temp}]}
    }

    result = create_router_bgp(tgen, topo, redist_dict)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify that R2's vrf RED routes are now imported into vrf "
        "default of R3, next-hop pointing to vrf GREEN"
    )
    for addr_type in ADDR_TYPES:
        static_routes = {
            "r3": {"static_routes": [{"network": [NETWORK2_1[addr_type]]}]}
        }
        next_hop_1 = topo["routers"]["r2"]["links"]["r3-link3"][addr_type].split("/")[0]
        result = verify_bgp_rib(
            tgen, addr_type, "r3", static_routes, next_hop=next_hop_1
        )
        assert result is True, "Testcase {} : Failed \n Error {}".format(
            tc_name, result
        )

        result = verify_rib(tgen, addr_type, "r3", static_routes, next_hop=next_hop_1)
        assert result is True, "Testcase {} : Failed \n Error {}".format(
            tc_name, result
        )

    write_test_footer(tc_name)


if __name__ == "__main__":
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))



#!/usr/bin/env python
# SPDX-License-Identifier: ISC

#
# test_bgp_vpnv4_noretain.py
# Part of NetDEF Topology Tests
#
# Copyright 2022 6WIND S.A.
#

"""
 test_bgp_vpnv4_noretain.py: Do not keep the VPNvx entries when no
 VRF matches incoming VPNVx entries
"""

import os
import sys
import json
from functools import partial
from copy import deepcopy
import pytest

# Save the Current Working Directory to find configuration files.
CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, "../"))

# pylint: disable=C0413
# Import topogen and topotest helpers
from lib import topotest
from lib.topogen import Topogen, TopoRouter, get_topogen
from lib.topolog import logger

# Required to instantiate the topology builder class.


pytestmark = [pytest.mark.bgpd]


def build_topo(tgen):
    "Build function"

    tgen.add_router("r1")
    tgen.add_router("r2")

    switch = tgen.add_switch("s1")
    switch.add_link(tgen.gears["r1"])
    switch.add_link(tgen.gears["r2"])

    switch = tgen.add_switch("s2")
    switch.add_link(tgen.gears["r1"])

    switch = tgen.add_switch("s3")
    switch.add_link(tgen.gears["r2"])

    switch = tgen.add_switch("s4")
    switch.add_link(tgen.gears["r2"])

    switch = tgen.add_switch("s5")
    switch.add_link(tgen.gears["r1"])

    switch = tgen.add_switch("s6")
    switch.add_link(tgen.gears["r2"])

    switch = tgen.add_switch("s7")
    switch.add_link(tgen.gears["r1"])

    switch = tgen.add_switch("s8")
    switch.add_link(tgen.gears["r2"])


def _populate_iface():
    tgen = get_topogen()
    cmds_list = [
        "modprobe mpls_router",
        "echo 100000 > /proc/sys/net/mpls/platform_labels",
        "ip link add vrf1 type vrf table 10",
        "ip link set dev vrf1 up",
        "ip link set dev {0}-eth1 master vrf1",
        "echo 1 > /proc/sys/net/mpls/conf/vrf1/input",
        "ip link add vrf2 type vrf table 20",
        "ip link set dev vrf2 up",
        "ip link set dev {0}-eth2 master vrf2",
        "echo 1 > /proc/sys/net/mpls/conf/vrf2/input",
        "ip link add vrf3 type vrf table 30",
        "ip link set dev vrf3 up",
        "ip link set dev {0}-eth3 master vrf3",
        "echo 1 > /proc/sys/net/mpls/conf/vrf3/input",
        "ip link add vrf4 type vrf table 40",
        "ip link set dev vrf4 up",
        "ip link set dev {0}-eth4 master vrf4",
        "echo 1 > /proc/sys/net/mpls/conf/vrf4/input",
    ]

    for cmd in cmds_list:
        input = cmd.format("r1", "1", "2")
        logger.info("input: " + cmd)
        output = tgen.net["r1"].cmd(cmd.format("r1", "1", "2"))
        logger.info("output: " + output)

    for cmd in cmds_list:
        input = cmd.format("r2", "2", "1")
        logger.info("input: " + cmd)
        output = tgen.net["r2"].cmd(cmd.format("r2", "2", "1"))
        logger.info("output: " + output)


def setup_module(mod):
    "Sets up the pytest environment"

    tgen = Topogen(build_topo, mod.__name__)
    tgen.start_topology()

    router_list = tgen.routers()
    _populate_iface()

    for rname, router in router_list.items():
        router.load_config(
            TopoRouter.RD_ZEBRA, os.path.join(CWD, "{}/zebra.conf".format(rname))
        )
        router.load_config(
            TopoRouter.RD_BGP, os.path.join(CWD, "{}/bgpd.conf".format(rname))
        )

    # Initialize all routers.
    tgen.start_router()


def teardown_module(_mod):
    "Teardown the pytest environment"
    tgen = get_topogen()

    tgen.stop_topology()


def router_json_cmp_exact_filter(router, cmd, expected):
    output = router.vtysh_cmd(cmd)
    logger.info("{}: {}\n{}".format(router.name, cmd, output))

    json_output = json.loads(output)

    # filter out tableVersion, version and nhVrfID
    json_output.pop("tableVersion")
    if "totalRoutes" in json_output:
        json_output.pop("totalRoutes")
    if "totalPaths" in json_output:
        json_output.pop("totalPaths")
    for rd, data in json_output["routes"]["routeDistinguishers"].items():
        for _, attrs in data.items():
            for attr in attrs:
                if "nhVrfId" in attr:
                    attr.pop("nhVrfId")
                if "version" in attr:
                    attr.pop("version")

    # filter out RD with no data (e.g. "444:3": {})
    json_tmp = deepcopy(json_output)
    for rd, data in json_tmp["routes"]["routeDistinguishers"].items():
        if len(data.keys()) == 0:
            json_output["routes"]["routeDistinguishers"].pop(rd)

    return topotest.json_cmp(json_output, expected, exact=True)


def router_vrf_json_cmp_exact_filter(router, cmd, expected):
    output = router.vtysh_cmd(cmd)
    logger.info("{}: {}\n{}".format(router.name, cmd, output))

    json_output = json.loads(output)

    print(json_output)

    # filter out tableVersion, version, nhVrfId and vrfId
    for vrf, data in json_output.items():
        if "vrfId" in data:
            data.pop("vrfId")
        if "tableVersion" in data:
            data.pop("tableVersion")
        if "totalRoutes" in data:
            data.pop("totalRoutes")
        if "totalPaths" in data:
            data.pop("totalPaths")
        if "routes" not in data:
            continue
        for _, attrs in data["routes"].items():
            for attr in attrs:
                if "nhVrfId" in attr:
                    attr.pop("nhVrfId")
                if "version" in attr:
                    attr.pop("version")

    # filter out VRF with no routes
    json_tmp = deepcopy(json_output)
    for vrf, data in json_tmp.items():
        if "routes" not in data or len(data["routes"].keys()) == 0:
            json_output.pop(vrf)

    return topotest.json_cmp(json_output, expected, exact=True)


def check_show_bgp_ipv4_vpn(rname, json_file):
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)
    router = tgen.gears[rname]

    logger.info("Checking VPNv4 routes for convergence on {}".format(rname))

    json_file = "{}/{}/{}".format(CWD, router.name, json_file)
    expected = json.loads(open(json_file).read())
    test_func = partial(
        router_json_cmp_exact_filter,
        router,
        "show bgp ipv4 vpn json",
        expected,
    )
    _, result = topotest.run_and_expect(test_func, None, count=20, wait=1)
    assertmsg = '"{}" JSON output mismatches'.format(router.name)
    assert result is None, assertmsg


def check_show_bgp_ipv4_vpn_peer_advertised_routes(rname, peer, json_file):
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)
    router = tgen.gears[rname]

    logger.info(
        "Checking VPNv4 advertised routes for on {} for peer {}".format(rname, peer)
    )

    json_file = "{}/{}/{}".format(CWD, router.name, json_file)
    expected = json.loads(open(json_file).read())
    test_func = partial(
        topotest.router_json_cmp,
        router,
        "show bgp ipv4 vpn neighbors {} advertised-routes detail json".format(peer),
        expected,
    )
    _, result = topotest.run_and_expect(test_func, None, count=20, wait=1)
    assertmsg = '"{}" JSON output mismatches'.format(router.name)
    assert result is None, assertmsg


def check_show_bgp_vrf_ipv4(rname, json_file):
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)
    router = tgen.gears[rname]

    logger.info("Checking VRF IPv4 routes for convergence on {}".format(rname))

    json_file = "{}/{}/{}".format(CWD, router.name, json_file)
    expected = json.loads(open(json_file).read())
    test_func = partial(
        router_vrf_json_cmp_exact_filter,
        router,
        "show bgp vrf all ipv4 unicast json",
        expected,
    )
    _, result = topotest.run_and_expect(test_func, None, count=20, wait=1)
    assertmsg = '"{}" JSON output mismatches'.format(router.name)
    assert result is None, assertmsg


def test_protocols_convergence_step0():
    """
    Assert that all protocols have converged
    """
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    # check that r2 peerings are ok
    logger.info("Checking BGP ipv4 vpn summary for r2")
    router = tgen.gears["r2"]
    json_file = "{}/{}/ipv4_vpn_summary.json".format(CWD, router.name)
    expected = json.loads(open(json_file).read())
    test_func = partial(
        topotest.router_json_cmp,
        router,
        "show bgp ipv4 vpn summary json",
        expected,
    )
    _, result = topotest.run_and_expect(test_func, None, count=20, wait=1)
    assertmsg = '"{}" JSON output mismatches'.format(router.name)
    assert result is None, assertmsg


def test_bgp_no_retain_step1():
    """
    Check bgp no retain route-target all on r1
    """

    rname = "r1"
    check_show_bgp_ipv4_vpn(rname, "ipv4_vpn_routes_no_retain_init.json")
    check_show_bgp_ipv4_vpn("r2", "ipv4_vpn_routes_all.json")

    check_show_bgp_vrf_ipv4(rname, "ipv4_vrf_all_routes_init.json")


def test_bgp_retain_step2():
    """
    Apply and check bgp retain route-target all on r1
    """
    rname = "r1"
    cfg = """
configure
router bgp 65500
 address-family ipv4 vpn
  bgp retain route-target all
"""

    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    router = tgen.gears[rname]
    router.vtysh_cmd(cfg)

    check_show_bgp_ipv4_vpn(rname, "ipv4_vpn_routes_all.json")
    check_show_bgp_ipv4_vpn("r2", "ipv4_vpn_routes_all.json")

    check_show_bgp_vrf_ipv4(rname, "ipv4_vrf_all_routes_init.json")


def test_bgp_no_retain_step3():
    """
    Apply and check no bgp retain route-target all on r1
    """
    rname = "r1"

    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    router = tgen.gears["r1"]
    router.vtysh_cmd(
        "configure\nrouter bgp 65500\naddress-family ipv4 vpn\nno bgp retain route-target all\n"
    )

    check_show_bgp_ipv4_vpn(rname, "ipv4_vpn_routes_no_retain_init.json")
    check_show_bgp_ipv4_vpn("r2", "ipv4_vpn_routes_all.json")

    check_show_bgp_vrf_ipv4(rname, "ipv4_vrf_all_routes_init.json")


def test_bgp_no_retain_add_vrf2_step4():
    """
    Add vrf2 on r1 and check bgp tables
    """

    rname = "r1"
    cfg = """
configure
router bgp 65500 vrf vrf2
 bgp router-id 192.0.2.1
 address-family ipv4 unicast
  redistribute connected
  label vpn export 102
  rd vpn export 192.0.2.1:200
  rt vpn import 192.0.2.2:200
  import vpn
 exit-address-family
!
"""
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    router = tgen.gears[rname]
    router.vtysh_cmd(cfg)

    check_show_bgp_ipv4_vpn(rname, "ipv4_vpn_routes_no_retain_init_plus_r2_vrf2.json")
    check_show_bgp_ipv4_vpn("r2", "ipv4_vpn_routes_all.json")

    check_show_bgp_vrf_ipv4(rname, "ipv4_vrf_all_routes_plus_r2_vrf2.json")


def test_bgp_no_retain_unimport_vrf2_step5():
    """
    Unimport to vrf2 on r1 and check bgp tables
    """

    rname = "r1"
    cfg = """
configure
router bgp 65500 vrf vrf2
 address-family ipv4 unicast
  no import vpn
 exit-address-family
!
"""
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    router = tgen.gears[rname]
    router.vtysh_cmd(cfg)

    check_show_bgp_ipv4_vpn(rname, "ipv4_vpn_routes_no_retain_init.json")
    check_show_bgp_ipv4_vpn("r2", "ipv4_vpn_routes_all.json")

    check_show_bgp_vrf_ipv4(rname, "ipv4_vrf_all_routes_init.json")


def test_bgp_no_retain_import_vrf2_step6():
    """
    Re-import to vrf2 on r1 and check bgp tables
    """

    rname = "r1"
    cfg = """
configure
router bgp 65500 vrf vrf2
 address-family ipv4 unicast
  import vpn
 exit-address-family
!
"""
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    router = tgen.gears[rname]
    router.vtysh_cmd(cfg)

    check_show_bgp_ipv4_vpn(rname, "ipv4_vpn_routes_no_retain_init_plus_r2_vrf2.json")
    check_show_bgp_ipv4_vpn("r2", "ipv4_vpn_routes_all.json")

    check_show_bgp_vrf_ipv4(rname, "ipv4_vrf_all_routes_plus_r2_vrf2.json")


def test_bgp_no_retain_import_vrf1_step7():
    """
    Import r1 vrf1 into r1 vrf2 and check bgp tables
    """

    rname = "r1"
    cfg = """
configure
router bgp 65500 vrf vrf2
 address-family ipv4 unicast
  rt vpn import 192.0.2.1:100
 exit-address-family
!
"""
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    router = tgen.gears[rname]
    router.vtysh_cmd(cfg)

    check_show_bgp_ipv4_vpn(rname, "ipv4_vpn_routes_no_retain_init.json")
    check_show_bgp_ipv4_vpn("r2", "ipv4_vpn_routes_all.json")

    check_show_bgp_vrf_ipv4(rname, "ipv4_vrf_all_routes_plus_r1_vrf1.json")


def test_bgp_no_retain_import_vrf3_step8():
    """
    Import r2 vrf3 into r1 vrf2 and check bgp tables
    """

    rname = "r1"
    cfg = """
configure
router bgp 65500 vrf vrf2
 address-family ipv4 unicast
  rt vpn import 192.0.2.2:300
 exit-address-family
!
"""
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    router = tgen.gears[rname]
    router.vtysh_cmd(cfg)

    check_show_bgp_ipv4_vpn(rname, "ipv4_vpn_routes_no_retain_init_plus_r2_vrf3.json")
    check_show_bgp_ipv4_vpn("r2", "ipv4_vpn_routes_all.json")

    check_show_bgp_vrf_ipv4(rname, "ipv4_vrf_all_routes_plus_r2_vrf3.json")


def test_bgp_no_retain_unimport_vrf3_step9():
    """
    Un-import r2 vrf3 into r1 vrf2 and check bgp tables
    """

    rname = "r1"
    cfg = """
configure
router bgp 65500 vrf vrf2
 address-family ipv4 unicast
  no rt vpn import 192.0.2.2:300
 exit-address-family
!
"""
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    router = tgen.gears[rname]
    router.vtysh_cmd(cfg)

    check_show_bgp_ipv4_vpn(rname, "ipv4_vpn_routes_no_retain_init.json")
    check_show_bgp_ipv4_vpn("r2", "ipv4_vpn_routes_all.json")

    check_show_bgp_vrf_ipv4(rname, "ipv4_vrf_all_routes_init.json")


def test_bgp_no_retain_import_vrf3_step10():
    """
    Import r2 vrf3 into r1 vrf2 and check bgp tables
    """

    rname = "r1"
    cfg = """
configure
router bgp 65500 vrf vrf2
 address-family ipv4 unicast
  rt vpn import 192.0.2.2:300
 exit-address-family
!
"""
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    router = tgen.gears[rname]
    router.vtysh_cmd(cfg)

    check_show_bgp_ipv4_vpn(rname, "ipv4_vpn_routes_no_retain_init_plus_r2_vrf3.json")
    check_show_bgp_ipv4_vpn("r2", "ipv4_vpn_routes_all.json")

    check_show_bgp_vrf_ipv4(rname, "ipv4_vrf_all_routes_plus_r2_vrf3.json")


def test_bgp_no_retain_remove_vrf2_step11():
    """
    Remove BGP vrf2 on r1 and check bgp tables
    """

    rname = "r1"
    cfg = """
configure
no router bgp 65500 vrf vrf2
"""
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    router = tgen.gears[rname]
    router.vtysh_cmd(cfg)

    check_show_bgp_ipv4_vpn(rname, "ipv4_vpn_routes_no_retain_init.json")
    check_show_bgp_ipv4_vpn("r2", "ipv4_vpn_routes_all.json")

    check_show_bgp_vrf_ipv4(rname, "ipv4_vrf_all_routes_init.json")


def test_bgp_retain_step12():
    """
    Configure retain and check bgp tables
    """

    rname = "r1"
    cfg = """
configure
router bgp 65500
 address-family ipv4 vpn
  bgp retain route-target all
"""
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    router = tgen.gears[rname]
    router.vtysh_cmd(cfg)

    check_show_bgp_ipv4_vpn(rname, "ipv4_vpn_routes_all.json")
    check_show_bgp_ipv4_vpn("r2", "ipv4_vpn_routes_all.json")

    check_show_bgp_vrf_ipv4(rname, "ipv4_vrf_all_routes_init.json")


def test_bgp_advertised_routes_step13():
    """
    Dump advertised routes from r1 to r2
    Check that the localpref attribute is set on the show command
    """

    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    check_show_bgp_ipv4_vpn_peer_advertised_routes(
        "r1", "10.125.0.2", "ipv4_vpn_routes_advertised_10_125_0_2.json"
    )


def test_memory_leak():
    "Run the memory leak test and report results."
    tgen = get_topogen()
    if not tgen.is_memleak_enabled():
        pytest.skip("Memory leak test/report is disabled")

    tgen.report_memory_leaks()


if __name__ == "__main__":
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))



#!/usr/bin/python
# SPDX-License-Identifier: ISC

#
# Copyright (c) 2020 by VMware, Inc. ("VMware")
# Used Copyright (c) 2018 by Network Device Education Foundation, Inc.
# ("NetDEF") in this file.
#


"""OSPF Basic Functionality Automation."""
import os
import sys
import time
import pytest

# Save the Current Working Directory to find configuration files.
CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, "../"))
sys.path.append(os.path.join(CWD, "../lib/"))

# pylint: disable=C0413
# Import topogen and topotest helpers
from lib.topogen import Topogen, get_topogen

# Import topoJson from lib, to create topology and initial configuration
from lib.common_config import (
    start_topology,
    write_test_header,
    write_test_footer,
    reset_config_on_routers,
    step,
    verify_rib,
    stop_router,
    start_router,
    create_static_routes,
    start_router_daemons,
    kill_router_daemons,
)

from lib.ospf import verify_ospf_neighbor, verify_ospf_rib, create_router_ospf

from lib.topolog import logger
from lib.topojson import build_config_from_json

pytestmark = [pytest.mark.ospfd, pytest.mark.staticd]

# Global variables
topo = None

NETWORK = {
    "ipv4": [
        "11.0.20.1/32",
        "11.0.20.2/32",
        "11.0.20.3/32",
        "11.0.20.4/32",
        "11.0.20.5/32",
    ]
}
"""
Topology:
      Please view in a fixed-width font such as Courier.
      +---+  A1       +---+
      +R1 +------------+R2 |
      +-+-+-           +--++
        |  --        --  |
        |    -- A0 --    |
      A0|      ----      |
        |      ----      | A2
        |    --    --    |
        |  --        --  |
      +-+-+-            +-+-+
      +R0 +-------------+R3 |
      +---+     A3     +---+

TESTCASES =
1. Verify ospf functionality after restart ospfd.
2. Verify ospf functionality after restart FRR service.
3. Verify ospf functionality when staticd is restarted.
 """


def setup_module(mod):
    """
    Sets up the pytest environment

    * `mod`: module name
    """
    testsuite_run_time = time.asctime(time.localtime(time.time()))
    logger.info("Testsuite start time: {}".format(testsuite_run_time))
    logger.info("=" * 40)

    logger.info("Running setup_module to create topology")

    # This function initiates the topology build with Topogen...
    json_file = "{}/ospf_chaos.json".format(CWD)
    tgen = Topogen(json_file, mod.__name__)
    global topo
    topo = tgen.json_topo
    # ... and here it calls Mininet initialization functions.

    # Starting topology, create tmp files which are loaded to routers
    #  to start daemons and then start routers
    start_topology(tgen)

    # Creating configuration from JSON
    build_config_from_json(tgen, topo)

    # Don't run this test if we have any failure.
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    ospf_covergence = verify_ospf_neighbor(tgen, topo)
    assert ospf_covergence is True, "setup_module :Failed \n Error  {}".format(
        ospf_covergence
    )

    logger.info("Running setup_module() done")


def teardown_module():
    """
    Teardown the pytest environment.

    * `mod`: module name
    """

    logger.info("Running teardown_module to delete topology")

    tgen = get_topogen()

    # Stop toplogy and Remove tmp files
    tgen.stop_topology()

    logger.info(
        "Testsuite end time: {}".format(time.asctime(time.localtime(time.time())))
    )
    logger.info("=" * 40)


# ##################################
# Test cases start here.
# ##################################
def test_ospf_chaos_tc31_p1(request):
    """Verify ospf functionality after restart ospfd."""
    tc_name = request.node.name
    write_test_header(tc_name)
    tgen = get_topogen()
    global topo
    step("Bring up the base config as per the topology")
    reset_config_on_routers(tgen)

    step(
        "Create static routes(10.0.20.1/32) in R1 and redistribute "
        "to OSPF using route map."
    )

    # Create Static routes
    input_dict = {
        "r0": {
            "static_routes": [
                {
                    "network": NETWORK["ipv4"][0],
                    "no_of_ip": 5,
                    "next_hop": "Null0",
                }
            ]
        }
    }
    result = create_static_routes(tgen, input_dict)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    ospf_red_r0 = {"r0": {"ospf": {"redistribute": [{"redist_type": "static"}]}}}
    result = create_router_ospf(tgen, topo, ospf_red_r0)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step("Verify OSPF neighbors after base config is done.")
    # Api call verify whether OSPF is converged
    ospf_covergence = verify_ospf_neighbor(tgen, topo)
    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
        ospf_covergence
    )

    step("Verify that route is advertised to R1.")
    dut = "r1"
    protocol = "ospf"
    nh = topo["routers"]["r0"]["links"]["r1"]["ipv4"].split("/")[0]
    result = verify_ospf_rib(tgen, dut, input_dict, next_hop=nh)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    result = verify_rib(tgen, "ipv4", dut, input_dict, protocol=protocol, next_hop=nh)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step("Kill OSPFd daemon on R0.")
    kill_router_daemons(tgen, "r0", ["ospfd"])

    step("Verify OSPF neighbors are down after killing ospfd in R0")
    dut = "r0"
    # Api call verify whether OSPF is converged
    ospf_covergence = verify_ospf_neighbor(tgen, topo, dut=dut, expected=False)
    assert ospf_covergence is not True, "Testcase Failed \n Error  {}".format(
        ospf_covergence
    )

    step("Verify that route advertised to R1 are deleted from RIB and FIB.")
    dut = "r1"
    protocol = "ospf"
    result = verify_ospf_rib(tgen, dut, input_dict, expected=False)
    assert (
        result is not True
    ), "Testcase {} : Failed \n  r1: OSPF routes are present \n Error: {}".format(
        tc_name, result
    )

    result = verify_rib(
        tgen, "ipv4", dut, input_dict, protocol=protocol, expected=False
    )
    assert (
        result is not True
    ), "Testcase {} : Failed \n  r1: routes are still present \n Error: {}".format(
        tc_name, result
    )

    step("Bring up OSPFd daemon on R0.")
    start_router_daemons(tgen, "r0", ["ospfd"])

    step("Verify OSPF neighbors are up after bringing back ospfd in R0")
    # Api call verify whether OSPF is converged
    ospf_covergence = verify_ospf_neighbor(tgen, topo)
    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
        ospf_covergence
    )

    step(
        "All the neighbours are up and routes are installed before the"
        " restart. Verify OSPF route table and ip route table."
    )
    dut = "r1"
    protocol = "ospf"
    result = verify_ospf_rib(tgen, dut, input_dict, next_hop=nh)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    result = verify_rib(tgen, "ipv4", dut, input_dict, protocol=protocol, next_hop=nh)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step("Kill OSPFd daemon on R1.")
    kill_router_daemons(tgen, "r1", ["ospfd"])

    step("Verify OSPF neighbors are down after killing ospfd in R1")
    dut = "r1"
    # Api call verify whether OSPF is converged
    ospf_covergence = verify_ospf_neighbor(tgen, topo, dut=dut, expected=False)
    assert ospf_covergence is not True, "Testcase Failed \n Error  {}".format(
        ospf_covergence
    )

    step("Bring up OSPFd daemon on R1.")
    start_router_daemons(tgen, "r1", ["ospfd"])

    step("Verify OSPF neighbors are up after bringing back ospfd in R1")
    # Api call verify whether OSPF is converged
    ospf_covergence = verify_ospf_neighbor(tgen, topo)
    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
        ospf_covergence
    )

    step(
        "All the neighbours are up and routes are installed before the"
        " restart. Verify OSPF route table and ip route table."
    )

    dut = "r1"
    protocol = "ospf"
    result = verify_ospf_rib(tgen, dut, input_dict, next_hop=nh)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    result = verify_rib(tgen, "ipv4", dut, input_dict, protocol=protocol, next_hop=nh)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    write_test_footer(tc_name)


def test_ospf_chaos_tc32_p1(request):
    """Verify ospf functionality after restart FRR service."""
    tc_name = request.node.name
    write_test_header(tc_name)
    tgen = get_topogen()
    global topo
    step("Bring up the base config as per the topology")
    reset_config_on_routers(tgen)

    step(
        "Create static routes(10.0.20.1/32) in R1 and redistribute "
        "to OSPF using route map."
    )

    # Create Static routes
    input_dict = {
        "r0": {
            "static_routes": [
                {
                    "network": NETWORK["ipv4"][0],
                    "no_of_ip": 5,
                    "next_hop": "Null0",
                }
            ]
        }
    }
    result = create_static_routes(tgen, input_dict)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    ospf_red_r0 = {"r0": {"ospf": {"redistribute": [{"redist_type": "static"}]}}}
    result = create_router_ospf(tgen, topo, ospf_red_r0)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step("Verify OSPF neighbors after base config is done.")
    # Api call verify whether OSPF is converged
    ospf_covergence = verify_ospf_neighbor(tgen, topo)
    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
        ospf_covergence
    )

    step("Verify that route is advertised to R1.")
    dut = "r1"
    protocol = "ospf"

    nh = topo["routers"]["r0"]["links"]["r1"]["ipv4"].split("/")[0]
    result = verify_ospf_rib(tgen, dut, input_dict, next_hop=nh)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    result = verify_rib(tgen, "ipv4", dut, input_dict, protocol=protocol, next_hop=nh)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step("Restart frr on R0")
    stop_router(tgen, "r0")
    start_router(tgen, "r0")

    step("Verify OSPF neighbors are up after restarting R0")
    # Api call verify whether OSPF is converged
    ospf_covergence = verify_ospf_neighbor(tgen, topo)
    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
        ospf_covergence
    )

    step(
        "All the neighbours are up and routes are installed before the"
        " restart. Verify OSPF route table and ip route table."
    )
    dut = "r1"
    protocol = "ospf"
    result = verify_ospf_rib(tgen, dut, input_dict, next_hop=nh)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    result = verify_rib(tgen, "ipv4", dut, input_dict, protocol=protocol, next_hop=nh)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step("Restart frr on R1")
    stop_router(tgen, "r1")
    start_router(tgen, "r1")

    step("Verify OSPF neighbors are up after restarting R1")
    # Api call verify whether OSPF is converged
    ospf_covergence = verify_ospf_neighbor(tgen, topo)
    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
        ospf_covergence
    )

    step(
        "All the neighbours are up and routes are installed before the"
        " restart. Verify OSPF route table and ip route table."
    )
    dut = "r1"
    protocol = "ospf"
    result = verify_ospf_rib(tgen, dut, input_dict, next_hop=nh)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    result = verify_rib(tgen, "ipv4", dut, input_dict, protocol=protocol, next_hop=nh)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    write_test_footer(tc_name)


def test_ospf_chaos_tc34_p1(request):
    """
    verify ospf functionality when staticd is restarted.

    Verify ospf functionalitywhen staticroutes are
    redistributed & Staticd is restarted.
    """
    tc_name = request.node.name
    write_test_header(tc_name)
    tgen = get_topogen()
    global topo
    step("Bring up the base config as per the topology")
    reset_config_on_routers(tgen)

    step(
        "Create static routes(10.0.20.1/32) in R1 and redistribute "
        "to OSPF using route map."
    )

    # Create Static routes
    input_dict = {
        "r0": {
            "static_routes": [
                {
                    "network": NETWORK["ipv4"][0],
                    "no_of_ip": 5,
                    "next_hop": "Null0",
                }
            ]
        }
    }
    result = create_static_routes(tgen, input_dict)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    ospf_red_r0 = {"r0": {"ospf": {"redistribute": [{"redist_type": "static"}]}}}
    result = create_router_ospf(tgen, topo, ospf_red_r0)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step("Verify OSPF neighbors after base config is done.")
    # Api call verify whether OSPF is converged
    ospf_covergence = verify_ospf_neighbor(tgen, topo)
    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
        ospf_covergence
    )

    step("Verify that route is advertised to R1.")
    dut = "r1"
    protocol = "ospf"
    nh = topo["routers"]["r0"]["links"]["r1"]["ipv4"].split("/")[0]
    result = verify_ospf_rib(tgen, dut, input_dict, next_hop=nh)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    result = verify_rib(tgen, "ipv4", dut, input_dict, protocol=protocol, next_hop=nh)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step("Kill staticd daemon on R0.")
    kill_router_daemons(tgen, "r0", ["staticd"])

    step("Verify that route advertised to R1 are deleted from RIB and FIB.")
    dut = "r1"
    protocol = "ospf"
    result = verify_ospf_rib(tgen, dut, input_dict, expected=False)
    assert (
        result is not True
    ), "Testcase {} : Failed \n  r1: OSPF routes are present \n Error: {}".format(
        tc_name, result
    )

    result = verify_rib(
        tgen, "ipv4", dut, input_dict, protocol=protocol, expected=False
    )
    assert (
        result is not True
    ), "Testcase {} : Failed \n  r1: routes are still present \n Error: {}".format(
        tc_name, result
    )

    step("Bring up staticd daemon on R0.")
    start_router_daemons(tgen, "r0", ["staticd"])

    step("Verify OSPF neighbors are up after bringing back ospfd in R0")
    # Api call verify whether OSPF is converged
    ospf_covergence = verify_ospf_neighbor(tgen, topo)
    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
        ospf_covergence
    )

    step(
        "All the neighbours are up and routes are installed before the"
        " restart. Verify OSPF route table and ip route table."
    )
    dut = "r1"
    protocol = "ospf"
    result = verify_ospf_rib(tgen, dut, input_dict, next_hop=nh)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    result = verify_rib(tgen, "ipv4", dut, input_dict, protocol=protocol, next_hop=nh)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step("Kill staticd daemon on R1.")
    kill_router_daemons(tgen, "r1", ["staticd"])

    step("Bring up staticd daemon on R1.")
    start_router_daemons(tgen, "r1", ["staticd"])

    step("Verify OSPF neighbors are up after bringing back ospfd in R1")
    # Api call verify whether OSPF is converged
    ospf_covergence = verify_ospf_neighbor(tgen, topo)
    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
        ospf_covergence
    )

    step(
        "All the neighbours are up and routes are installed before the"
        " restart. Verify OSPF route table and ip route table."
    )

    dut = "r1"
    protocol = "ospf"
    result = verify_ospf_rib(tgen, dut, input_dict, next_hop=nh)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    result = verify_rib(tgen, "ipv4", dut, input_dict, protocol=protocol, next_hop=nh)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    write_test_footer(tc_name)


if __name__ == "__main__":
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))



#!/usr/bin/env python
# SPDX-License-Identifier: ISC

#
# test_ldp_vpls_topo1.py
# Part of NetDEF Topology Tests
#
# Copyright (c) 2017 by
# Network Device Education Foundation, Inc. ("NetDEF")
#

"""
test_ldp_vpls_topo1.py:

                   +---------+                +---------+
                   |         |                |         |
                   |   CE1   |                |   CE2   |
                   |         |                |         |
                   +---------+                +---------+
ce1-eth0 (172.16.1.1/24)|                          |ce2-eth0 (172.16.1.2/24)
                        |                          |
                        |                          |
                rt1-eth0|                          |rt2-eth0
                   +---------+  10.0.1.0/24   +---------+
                   |         |rt1-eth1        |         |
                   |   RT1   +----------------+   RT2   |
                   | 1.1.1.1 |        rt2-eth1| 2.2.2.2 |
                   |         |                |         |
                   +---------+                +---------+
                rt1-eth2|                          |rt2-eth2
                        |                          |
                        |                          |
             10.0.2.0/24|        +---------+       |10.0.3.0/24
                        |        |         |       |
                        |        |   RT3   |       |
                        +--------+ 3.3.3.3 +-------+
                         rt3-eth2|         |rt3-eth1
                                 +---------+
                                      |rt3-eth0
                                      |
                                      |
              ce3-eth0 (172.16.1.3/24)|
                                 +---------+
                                 |         |
                                 |   CE3   |
                                 |         |
                                 +---------+
"""

import os
import sys
import pytest
import json
from functools import partial

# Save the Current Working Directory to find configuration files.
CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, "../"))

# pylint: disable=C0413
# Import topogen and topotest helpers
from lib import topotest
from lib.topogen import Topogen, TopoRouter, get_topogen
from lib.topolog import logger

# Required to instantiate the topology builder class.

pytestmark = [pytest.mark.ldpd, pytest.mark.ospfd]


def build_topo(tgen):
    "Build function"

    #
    # Define FRR Routers
    #
    for router in ["ce1", "ce2", "ce3", "r1", "r2", "r3"]:
        tgen.add_router(router)

    #
    # Define connections
    #
    switch = tgen.add_switch("s1")
    switch.add_link(tgen.gears["ce1"])
    switch.add_link(tgen.gears["r1"])

    switch = tgen.add_switch("s2")
    switch.add_link(tgen.gears["ce2"])
    switch.add_link(tgen.gears["r2"])

    switch = tgen.add_switch("s3")
    switch.add_link(tgen.gears["ce3"])
    switch.add_link(tgen.gears["r3"])

    switch = tgen.add_switch("s4")
    switch.add_link(tgen.gears["r1"])
    switch.add_link(tgen.gears["r2"])

    switch = tgen.add_switch("s5")
    switch.add_link(tgen.gears["r1"])
    switch.add_link(tgen.gears["r3"])

    switch = tgen.add_switch("s6")
    switch.add_link(tgen.gears["r2"])
    switch.add_link(tgen.gears["r3"])


def setup_module(mod):
    "Sets up the pytest environment"
    tgen = Topogen(build_topo, mod.__name__)
    tgen.start_topology()

    router_list = tgen.routers()

    # For all registered routers, load the zebra configuration file
    for rname, router in router_list.items():
        router.load_config(
            TopoRouter.RD_ZEBRA, os.path.join(CWD, "{}/zebra.conf".format(rname))
        )
        # Don't start ospfd and ldpd in the CE nodes
        if router.name[0] == "r":
            router.load_config(
                TopoRouter.RD_OSPF, os.path.join(CWD, "{}/ospfd.conf".format(rname))
            )
            router.load_config(
                TopoRouter.RD_LDP, os.path.join(CWD, "{}/ldpd.conf".format(rname))
            )

    tgen.start_router()


def teardown_module():
    "Teardown the pytest environment"
    tgen = get_topogen()

    # This function tears down the whole topology.
    tgen.stop_topology()


def router_compare_json_output(rname, command, reference, count=80, wait=1):
    "Compare router JSON output"

    logger.info('Comparing router "%s" "%s" output', rname, command)

    tgen = get_topogen()
    filename = "{}/{}/{}".format(CWD, rname, reference)
    expected = json.loads(open(filename).read())

    # Run test function until we get an result.
    test_func = partial(topotest.router_json_cmp, tgen.gears[rname], command, expected)
    _, diff = topotest.run_and_expect(test_func, None, count, wait)
    assertmsg = '"{}" JSON output mismatches the expected result'.format(rname)
    assert diff is None, assertmsg


def test_ospf_convergence():
    logger.info("Test: check OSPF adjacencies")
    tgen = get_topogen()

    # Skip if previous fatal error condition is raised
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    for rname in ["r1", "r2", "r3"]:
        router_compare_json_output(
            rname, "show ip ospf neighbor json", "show_ip_ospf_neighbor.json"
        )


def test_rib():
    logger.info("Test: verify RIB")
    tgen = get_topogen()

    # Skip if previous fatal error condition is raised
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    for rname in ["r1", "r2", "r3"]:
        router_compare_json_output(rname, "show ip route json", "show_ip_route.ref")


def test_ldp_adjacencies():
    logger.info("Test: verify LDP adjacencies")
    tgen = get_topogen()

    # Skip if previous fatal error condition is raised
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    for rname in ["r1", "r2", "r3"]:
        router_compare_json_output(
            rname, "show mpls ldp discovery json", "show_ldp_discovery.ref"
        )


def test_ldp_neighbors():
    logger.info("Test: verify LDP neighbors")
    tgen = get_topogen()

    # Skip if previous fatal error condition is raised
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    for rname in ["r1", "r2", "r3"]:
        router_compare_json_output(
            rname, "show mpls ldp neighbor json", "show_ldp_neighbor.ref"
        )


def test_ldp_bindings():
    logger.info("Test: verify LDP bindings")
    tgen = get_topogen()

    # Skip if previous fatal error condition is raised
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    for rname in ["r1", "r2", "r3"]:
        router_compare_json_output(
            rname, "show mpls ldp binding json", "show_ldp_binding.ref"
        )


def test_ldp_pwid_bindings():
    logger.info("Test: verify LDP PW-ID bindings")
    tgen = get_topogen()

    # Skip if previous fatal error condition is raised
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    for rname in ["r1", "r2", "r3"]:
        router_compare_json_output(
            rname, "show l2vpn atom binding json", "show_l2vpn_binding.ref"
        )


def test_ldp_pseudowires():
    logger.info("Test: verify LDP pseudowires")
    tgen = get_topogen()

    # Skip if previous fatal error condition is raised
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    for rname in ["r1", "r2", "r3"]:
        router_compare_json_output(
            rname, "show l2vpn atom vc json", "show_l2vpn_vc.ref"
        )


def test_ldp_pseudowires_after_link_down():
    logger.info("Test: verify LDP pseudowires after r1-r2 link goes down")
    tgen = get_topogen()

    # Skip if previous fatal error condition is raised
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    # Shut down r1-r2 link */
    tgen = get_topogen()
    rname = "r1"
    tgen.gears[rname].peer_link_enable("r1-eth1", False)
    router_compare_json_output(
        rname,
        "show ip route json",
        "show_ip_route_after_link_down.ref",
        count=160,
        wait=1,
    )
    # check if the pseudowire is still up (using an alternate path
    # for nexthop resolution). Give some extra wait time.
    for rname in ["r1", "r2", "r3"]:
        router_compare_json_output(
            rname, "show l2vpn atom vc json", "show_l2vpn_vc.ref", count=160, wait=1
        )


# Memory leak test template
def test_memory_leak():
    "Run the memory leak test and report results."
    tgen = get_topogen()
    if not tgen.is_memleak_enabled():
        pytest.skip("Memory leak test/report is disabled")

    tgen.report_memory_leaks()


if __name__ == "__main__":
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))



#!/usr/bin/env python
# SPDX-License-Identifier: ISC

#
# Part of NetDEF Topology Tests
#
# Copyright 2021 by LINE Corporation, Hiroki Shirokura <hiroki.shirokura@linecorp.com>
# Copyright 2023 6WIND S.A.

"""
test_isis_sr_flex_algo_topo2.py:

[+] Flex-Algos 128
[+] Flex-Algos 129
[+] Flex-Algos 130 include-any blue

            +--------+                  +--------+
            |        |                  |        |
            |  RT1   |------------------|  RT2   |
            |        |                  |        |
            +--------+                  +--------+
           /     |    \\                     |    \\
          /      |     \\                    |     \\
+--------+       |      \\                   |      \\
|        |       |       +--------+          |       +--------+
|  RT0   |       |       |        |          |       |        |
|        |       |       |  RT4   |------------------|  RT3   |
+--------+       |       |        |          |       |        |
          \\     |       +--------+          |       +--------+
           \\    |           |               |            |    \\
            +--------+       |          +--------+        |     \\
            |        |       |          |        |        |      +--------+
            |  RT5   |-------|----------|  RT6   |        |      |        |
            |        |       |          |        |        |      |  RT9   |
            +--------+       |          +--------+        |      |        |
                      \\     |                    \\      |      +--------+
                       \\    |                     \\     |     /
                        \\   |                      \\    |    /
                         +--------+                  +--------+
                         |        |                  |        |
                         |  RT8   |------------------|  RT7   |
                         |        |                  |        |
                         +--------+                  +--------+
"""

import os
import sys
import pytest
import json
from functools import partial

# Save the Current Working Directory to find configuration files.
CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, "../"))

# pylint: disable=C0413
# Import topogen and topotest helpers
from lib import topotest
from lib.topogen import Topogen, TopoRouter, get_topogen
from lib.topolog import logger


pytestmark = [pytest.mark.isisd]


def build_topo(tgen):
    "Build function"

    for i in range(0, 10):
        rt = tgen.add_router("rt{}".format(i))
        rt.run("sysctl -w net.ipv4.fib_multipath_hash_policy=1")

    def connect_routers(tgen, left_idx, right_idx):
        left = "rt{}".format(left_idx)
        right = "rt{}".format(right_idx)
        switch = tgen.add_switch("s-{}-{}".format(left, right))
        switch.add_link(tgen.gears[left], nodeif="eth-{}".format(right))
        switch.add_link(tgen.gears[right], nodeif="eth-{}".format(left))
        l_addr = "52:54:00:{}:{}:{}".format(left_idx, right_idx, left_idx)
        tgen.gears[left].run("ip link set eth-{} down".format(right))
        tgen.gears[left].run("ip link set eth-{} address {}".format(right, l_addr))
        tgen.gears[left].run("ip link set eth-{} up".format(right))
        tgen.gears[left].run("sysctl -w net.mpls.conf.eth-{}.input=1".format(right))
        r_addr = "52:54:00:{}:{}:{}".format(left_idx, right_idx, right_idx)
        tgen.gears[right].run("ip link set eth-{} down".format(left))
        tgen.gears[right].run("ip link set eth-{} address {}".format(left, r_addr))
        tgen.gears[right].run("ip link set eth-{} up".format(left))
        tgen.gears[right].run("sysctl -w net.mpls.conf.eth-{}.input=1".format(left))

    connect_routers(tgen, 0, 1)
    connect_routers(tgen, 0, 5)
    connect_routers(tgen, 1, 2)
    connect_routers(tgen, 1, 4)
    connect_routers(tgen, 1, 5)
    connect_routers(tgen, 2, 3)
    connect_routers(tgen, 2, 6)
    connect_routers(tgen, 3, 4)
    connect_routers(tgen, 3, 7)
    connect_routers(tgen, 3, 9)
    connect_routers(tgen, 4, 8)
    connect_routers(tgen, 5, 6)
    connect_routers(tgen, 5, 8)
    connect_routers(tgen, 6, 7)
    connect_routers(tgen, 7, 8)
    connect_routers(tgen, 7, 9)


def setup_module(mod):
    "Sets up the pytest environment"
    tgen = Topogen(build_topo, mod.__name__)
    frrdir = tgen.config.get(tgen.CONFIG_SECTION, "frrdir")
    if not os.path.isfile(os.path.join(frrdir, "pathd")):
        pytest.skip("pathd daemon wasn't built")
    tgen.start_topology()
    router_list = tgen.routers()

    # For all registered routers, load the zebra configuration file
    for rname, router in router_list.items():
        router.load_config(
            TopoRouter.RD_ZEBRA, os.path.join(CWD, "{}/zebra.conf".format(rname))
        )
        router.load_config(
            TopoRouter.RD_ISIS, os.path.join(CWD, "{}/isisd.conf".format(rname))
        )
        if rname in ["rt0", "rt9"]:
            router.load_config(
                TopoRouter.RD_BGP, os.path.join(CWD, "{}/bgpd.conf".format(rname))
            )
            router.load_config(
                TopoRouter.RD_PATH, os.path.join(CWD, "{}/pathd.conf".format(rname))
            )
            router.run("ip link add dum0 type dummy")
            router.run("ip link set dum0 up")
            if rname == "rt0":
                router.run("ip addr add 10.255.0.1/24 dev dum0")
            elif rname == "rt9":
                router.run("ip addr add 10.255.9.1/24 dev dum0")
    tgen.start_router()


def teardown_module():
    "Teardown the pytest environment"
    tgen = get_topogen()
    tgen.stop_topology()


def setup_testcase(msg):
    logger.info(msg)
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)
    return tgen


def open_json_file(filename):
    try:
        with open(filename, "r") as f:
            return json.load(f)
    except IOError:
        assert False, "Could not read file {}".format(filename)


def check_rib(name, cmd, expected_file):
    def _check(name, cmd, expected_file):
        logger.info("polling")
        tgen = get_topogen()
        router = tgen.gears[name]
        output = json.loads(router.vtysh_cmd(cmd))
        expected = open_json_file("{}/{}".format(CWD, expected_file))
        return topotest.json_cmp(output, expected)

    logger.info('[+] check {} "{}" {}'.format(name, cmd, expected_file))
    tgen = get_topogen()
    func = partial(_check, name, cmd, expected_file)
    _, result = topotest.run_and_expect(func, None, count=120, wait=0.5)
    assert result is None, "Failed"


def test_rib():
    check_rib("rt0", "show mpls table json", "rt0/step1/route.json")
    check_rib("rt1", "show mpls table json", "rt1/step1/route.json")
    check_rib("rt2", "show mpls table json", "rt2/step1/route.json")
    check_rib("rt3", "show mpls table json", "rt3/step1/route.json")
    check_rib("rt4", "show mpls table json", "rt4/step1/route.json")
    check_rib("rt5", "show mpls table json", "rt5/step1/route.json")
    check_rib("rt6", "show mpls table json", "rt6/step1/route.json")
    check_rib("rt7", "show mpls table json", "rt7/step1/route.json")
    check_rib("rt8", "show mpls table json", "rt8/step1/route.json")
    check_rib("rt9", "show mpls table json", "rt9/step1/route.json")


if __name__ == "__main__":
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))
